package de.ozgcloud.xta.test.app.data;

import java.time.ZonedDateTime;
import java.util.HashMap;
import java.util.Optional;

import lombok.RequiredArgsConstructor;
import org.springframework.stereotype.Component;

import de.ozgcloud.xta.test.app.model.XtaMessageMetaData;
import de.ozgcloud.xta.test.app.model.XtaTransportReport;

@Component
@RequiredArgsConstructor
public class XtaTransportReportRepository {
	private final HashMap<String, XtaMessageMetaData> messageIdTomessageMetaData = new HashMap<>();
	private final HashMap<String, XtaTransportReport.MessageStatus> messageIdToStatus = new HashMap<>();

	public boolean add(XtaMessageMetaData messageMetaData) {
		if (messageIdTomessageMetaData.containsKey(messageMetaData.messageId())) {
			return false;
		}
		messageIdTomessageMetaData.put(messageMetaData.messageId(), messageMetaData);
		messageIdToStatus.put(messageMetaData.messageId(), XtaTransportReport.MessageStatus.OFFEN);
		return true;
	}

	public boolean setClosedStatus(String messageId) {
		if (!messageIdToStatus.containsKey(messageId)) {
			return false;
		}

		messageIdToStatus.put(messageId, XtaTransportReport.MessageStatus.GRUEN);
		return true;

	}

	public Optional<XtaTransportReport> get(String messageId) {
		if (messageIdTomessageMetaData.containsKey(messageId)) {
			return Optional.of(XtaTransportReport.builder()
					.messageMetaData(messageIdTomessageMetaData.get(messageId))
					.status(messageIdToStatus.get(messageId))
					.reportTime(getNow())
					.build());
		}
		return Optional.empty();
	}

	private ZonedDateTime getNow() {
		return ZonedDateTime.now();
	}
}
