package de.ozgcloud.xta.test.app.data;

import static org.assertj.core.api.Assertions.assertThat;

import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.Spy;

import de.ozgcloud.xta.test.app.factory.XtaMessageMetaDataTestFactory;
import de.ozgcloud.xta.test.app.model.XtaMessageMetaData;
import de.ozgcloud.xta.test.app.model.XtaTransportReport;

public class XtaTransportReportRepositoryTest {
	@Spy
	private XtaTransportReportRepository repository;

	private final XtaMessageMetaData messageMetaData = XtaMessageMetaDataTestFactory.create();

	@Nested
	class Add{
		@Test
		void shouldReturnTrue() {
			var result = repository.add(messageMetaData);

			assertThat(result).isTrue();
		}

		@Test
		void shouldReturnFalseForDuplicateMessageId() {
			repository.add(messageMetaData);
			var result = repository.add(messageMetaData);

			assertThat(result).isFalse();
		}

	}

	@Nested
	class Get {
		@Nested
		class WithExistingMessageId {
			@Test
			void shouldReturnReport() {
				repository.add(messageMetaData);

				var result = repository.get(messageMetaData.messageId());

				assertThat(result).isPresent();
				assertThat(result.get().messageMetaData()).isEqualTo(messageMetaData);
			}

			@Test
			void shouldSetReportTime() {
				repository.add(messageMetaData);

				var result = repository.get(messageMetaData.messageId());

				assertThat(result).isPresent();
				assertThat(result.get().reportTime()).isNotNull();
			}

			@Test
			void shouldSetStatusOffen() {
				repository.add(messageMetaData);

				var result = repository.get(messageMetaData.messageId());

				assertThat(result).isPresent();
				assertThat(result.get().status()).isEqualTo(XtaTransportReport.MessageStatus.OFFEN);
			}
		}

		@Nested
		class WithMissingMessageId {
			@Test
			void shouldReturnEmpty() {
				var result = repository.get(messageMetaData.messageId());

				assertThat(result).isEmpty();
			}
		}

	}

	@Nested
	class SetClosedStatus {
		@Test
		void shouldSetClosedStatus() {
			repository.add(messageMetaData);

			repository.setClosedStatus(messageMetaData.messageId());
			var report = repository.get(messageMetaData.messageId());

			assertThat(report).isPresent();
			assertThat(report.get().status()).isEqualTo(XtaTransportReport.MessageStatus.GRUEN);
		}

		@Test
		void shouldReturnTrueOnSuccess() {
			repository.add(messageMetaData);

			var result = repository.setClosedStatus(messageMetaData.messageId());

			assertThat(result).isTrue();
		}

		@Test
		void shouldReturnFalseOnMissingMessageId() {
			var result = repository.setClosedStatus(messageMetaData.messageId());

			assertThat(result).isFalse();
		}

	}
}
